@@ -3,7 +3,7 @@ use std::ops::{Bound, RangeBounds};
 use std::sync::Arc;
 
 use crate::error::Result;
-use crate::file::{BlockHeader, TemporaryBlockFile};
+use crate::file::{BlockHeader, FixedSizeTupleFile, TupleFile, VariableSizeTupleFile};
 use crate::{create_mmap, BtreeConfig, Error};
 use binary_layout::prelude::*;
 use memmap2::MmapMut;
@@ -17,7 +17,6 @@ pub const MAX_NUMBER_KEYS: usize = 169;
 const MAX_NUMBER_CHILD_NODES: usize = MAX_NUMBER_KEYS + 1;
 
 // Defines a single BTree node with references to the actual values in a tuple file
-// This node can up to 1361 keys and 1362 child node references.
 define_layout!(node, LittleEndian, {
     id: u64,
     num_keys: u64,
@@ -30,7 +29,7 @@ define_layout!(node, LittleEndian, {
 pub struct NodeFile<K> {
     free_space_offset: usize,
     mmap: MmapMut,
-    keys: TemporaryBlockFile<K>,
+    keys: Box<dyn TupleFile<K>>,
 }
 
 pub enum SearchResult {
@@ -46,19 +45,32 @@ pub enum StackEntry {
 
 impl<K> NodeFile<K>
 where
-    K: Serialize + DeserializeOwned + Clone + Ord,
+    K: 'static + Serialize + DeserializeOwned + Clone + Ord + Send + Sync,
 {
+    /// Create a new file with the given capacity in number of keys.
     pub fn with_capacity(capacity: usize, config: &BtreeConfig) -> Result<NodeFile<K>> {
-        // Create an anonymous memory mapped file with the capacity as size
-        let capacity = capacity.max(1);
-        let mmap = create_mmap(capacity * NODE_BLOCK_ALIGNED_SIZE)?;
+        // Calculate the number of nodes based on the number of keys each node can hold
+        let capacity_in_nodes = num_integer::div_ceil(capacity, MAX_NUMBER_KEYS);
+        let capacity_in_nodes = capacity_in_nodes.max(1);
 
-        // Each node can hold 1361 keys, so we need the space for them as well
-        let number_of_keys = capacity * 1361;
-        let keys = TemporaryBlockFile::with_capacity(
-            (number_of_keys * config.est_max_value_size) + BlockHeader::size(),
-            config.block_cache_size,
-        )?;
+        // Create an anonymous memory mapped file that can hold the
+        let mmap = create_mmap(capacity_in_nodes * NODE_BLOCK_ALIGNED_SIZE)?;
+
+        // Create a tuple file that can hold the actual key values
+        let keys: Box<dyn TupleFile<K>> = match config.key_size {
+            super::TypeSize::Estimated(est_max_key_size) => {
+                let f = VariableSizeTupleFile::with_capacity(
+                    capacity * (est_max_key_size + BlockHeader::size()),
+                    config.block_cache_size,
+                )?;
+                Box::new(f)
+            }
+            super::TypeSize::Fixed(fixed_key_size) => {
+                let f =
+                    FixedSizeTupleFile::with_capacity(capacity * fixed_key_size, fixed_key_size)?;
+                Box::new(f)
+            }
+        };
 
         Ok(NodeFile {
             mmap,
@@ -66,7 +78,12 @@ where
             free_space_offset: 0,
         })
     }
+}
 
+impl<'a, K> NodeFile<K>
+where
+    K: Serialize + DeserializeOwned + Clone + Ord + Send + Sync,
+{
     /// Allocate a new node.
     ///
     /// Returns the ID of the new node.
@@ -270,25 +287,55 @@ where
     }
 
     pub fn get_key(&self, node_id: u64, i: usize) -> Result<Arc<K>> {
+        let key_id = self.get_key_id(node_id, i)?;
+        let result = self.keys.get(key_id.try_into()?)?;
+        Ok(result)
+    }
+
+    pub fn get_key_id(&self, node_id: u64, i: usize) -> Result<u64> {
         let view = self.get(node_id)?;
         let n: usize = view.num_keys().read() as usize;
         if i < n && i < MAX_NUMBER_KEYS {
             let offset = i * 8;
             let key_id: u64 =
                 u64::from_le_bytes(view.keys().data()[offset..(offset + 8)].try_into()?);
-            let result = self.keys.get(key_id.try_into()?)?;
-            Ok(result)
+            Ok(key_id)
         } else {
             Err(Error::KeyIndexOutOfBounds { idx: i, len: n })
         }
     }
 
-    pub fn set_key(&mut self, node_id: u64, i: usize, key: &K) -> Result<()> {
+    /// Sets the key for the given index `i` in the node `node_id`.
+    pub fn set_key_id(&mut self, node_id: u64, i: usize, key_id: u64) -> Result<()> {
+        let n: usize = self.get(node_id)?.num_keys().read() as usize;
+        if i <= n && i < MAX_NUMBER_KEYS {
+            let offset = i * 8;
+
+            let key_id = key_id.to_le_bytes();
+            let mut view = self.get_mut(node_id)?;
+
+            view.keys_mut().data_mut()[offset..(offset + 8)].copy_from_slice(&key_id);
+
+            if i == n {
+                // The key was inserted at the end of the list
+                let mut view = self.get_mut(node_id)?;
+                let n: u64 = (n + 1).try_into()?;
+                view.num_keys_mut().write(n);
+            }
+            Ok(())
+        } else {
+            Err(Error::KeyIndexOutOfBounds { idx: i, len: n })
+        }
+    }
+
+    /// Sets the key value for the given index `i` in the node `node_id`.
+    /// This will allocate a new block for the key.
+    pub fn set_key_value(&mut self, node_id: u64, i: usize, key: &K) -> Result<()> {
         let n: usize = self.get(node_id)?.num_keys().read() as usize;
         if i <= n && i < MAX_NUMBER_KEYS {
             let offset = i * 8;
             let key_size: usize = self.keys.serialized_size(key)?.try_into()?;
-            let key_id = self.keys.allocate_block(key_size + BlockHeader::size())?;
+            let key_id = self.keys.allocate_block(key_size)?;
             self.keys.put(key_id, key)?;
 
             let key_id: u64 = key_id.try_into()?;
@@ -404,7 +451,7 @@ where
 
         // The last element of the existing node is dangling without a child node,
         // use it as the key for the parent node
-        let split_key = self.get_key(existing_node, split_at - 1)?;
+        let split_key = self.get_key_id(existing_node, split_at - 1)?;
         let split_payload = self.get_payload(existing_node, split_at - 1)?;
         let mut existing_node_view = self.get_mut(existing_node)?;
         existing_node_view
@@ -413,11 +460,7 @@ where
 
         // Make space for the new entry in the parent node
         for i in ((child_idx + 1)..=self.number_of_keys(parent_node_id)?).rev() {
-            self.set_key(
-                parent_node_id,
-                i,
-                self.get_key(parent_node_id, i - 1)?.as_ref(),
-            )?;
+            self.set_key_id(parent_node_id, i, self.get_key_id(parent_node_id, i - 1)?)?;
             self.set_payload(parent_node_id, i, self.get_payload(parent_node_id, i - 1)?)?;
         }
         for i in ((child_idx + 1)..=self.number_of_children(parent_node_id)?).rev() {
@@ -429,7 +472,7 @@ where
         }
 
         // Insert the new child entry, the key and the payload into the parent node
-        self.set_key(parent_node_id, child_idx, &split_key)?;
+        self.set_key_id(parent_node_id, child_idx, split_key)?;
         self.set_payload(parent_node_id, child_idx, split_payload)?;
         self.set_child_node(parent_node_id, child_idx + 1, new_node_id)?;
 
@@ -443,7 +486,7 @@ where
 
         // The last element of the previous root node is dangling without a child node,
         // use it as the key for the parent node
-        let split_key = self.get_key(old_root_id, split_at - 1)?;
+        let split_key = self.get_key_id(old_root_id, split_at - 1)?;
         let split_payload = self.get_payload(old_root_id, split_at - 1)?;
         let mut existing_node_view = self.get_mut(old_root_id)?;
         existing_node_view
@@ -451,7 +494,7 @@ where
             .write((split_at - 1).try_into()?);
 
         // Insert the new child entry, the key and the payload into the parent node
-        self.set_key(new_root_id, 0, &split_key)?;
+        self.set_key_id(new_root_id, 0, split_key)?;
         self.set_payload(new_root_id, 0, split_payload)?;
         self.set_child_node(new_root_id, 0, old_root_id)?;
         self.set_child_node(new_root_id, 1, new_node_id)?;
@@ -467,10 +510,10 @@ where
 
             // Copy the right half of the keys, payload and child nodes to the new node
             for i in split_at..n {
-                self.set_key(
+                self.set_key_id(
                     target_node_id,
                     i - split_at,
-                    self.get_key(source_node_id, i)?.as_ref(),
+                    self.get_key_id(source_node_id, i)?,
                 )?;
                 self.set_payload(
                     target_node_id,
