@@ -19,6 +19,10 @@ pub enum Error {
     InsertFoundInternalNode,
     #[error("Splitting a node resulted in an empty child node.")]
     EmptyChildNodeInSplit,
+    #[error("The given capacity of {capacity} was invalid.")]
+    InvalidCapacity { capacity: usize },
+    #[error("Deserialization of block failed: {0}")]
+    DeserializeBlock(String),
     #[error(transparent)]
     IO(#[from] std::io::Error),
     #[error(transparent)]
